#!/usr/bin/env python
'''
This file contains Python command example for the PORT module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of different port attributes.
'''
import sys
import errno, sys
from python_sdk_api.sx_api import *

#string constants
TYPE = "TYPE"
SIZE = "SIZE"
IS_LOSSY = "LOSSY"
PG = "PG"
TC = "TC"
SP = "SP"
POOL_ID = "POOL_ID"
XON = "XON"
XOFF = "XOFF"
OVERRIDE_DEFAULT = "OVERRIDE_DEFAULT"
LATENCY = "LATENCY"

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

log_port = 0x10100

def get_buff_stat():
    stat_p = new_sx_buffer_status_t_p()
    
    rc = sx_api_cos_buff_status_get(handle, stat_p)
    print ("sx_api_cos_buff_status_get [rc=%d] " % (rc) )
    if rc != 0:
        sys.exit(errno.EACCES)   
        
    stat = sx_buffer_status_t_p_value(stat_p)
 
    print ("Ingress freespace [%d]" % stat.free_size_ingress)
    print ("Egress freespace [%d]" % stat.free_size_egress)
    
def cos_sb_buff_set(cmd, attr_params_list):
    port_buffer_attr_cnt = len(attr_params_list)
    port_buffer_attr_list_p = new_sx_cos_port_buffer_attr_t_arr(port_buffer_attr_cnt)

    i = 0
    for attr in attr_params_list:   
        attr_item_min = sx_cos_port_buffer_attr_t_arr_getitem(port_buffer_attr_list_p, i)
        if attr[TYPE]==SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
            attr_item_min.type = SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E
            attr_item_min.attr.ingress_port_pg_buff_attr.pg = attr[PG]
            attr_item_min.attr.ingress_port_pg_buff_attr.pool_id = attr[POOL_ID]
            
            if cmd == SX_ACCESS_CMD_SET:  
                attr_item_min.attr.ingress_port_pg_buff_attr.is_lossy = attr[IS_LOSSY]                  
                if attr[IS_LOSSY]==True:
                    attr_item_min.attr.ingress_port_pg_buff_attr.xon = 0
                    attr_item_min.attr.ingress_port_pg_buff_attr.xoff = 0
                    
                    pipe_latency=sx_cos_buff_pipeline_latency_t()    
                    if attr[OVERRIDE_DEFAULT]==True:
                        pipe_latency.override_default = True
                        pipe_latency.size = attr[LATENCY] 
                    else:
                            pipe_latency.override_default = False
                            pipe_latency.size = 0 
                    attr_item_min.attr.ingress_port_pg_buff_attr.pipeline_latency = pipe_latency
                else:
                    attr_item_min.attr.ingress_port_pg_buff_attr.xon = attr[XON]
                    attr_item_min.attr.ingress_port_pg_buff_attr.xoff = attr[XOFF]
                
                attr_item_min.attr.ingress_port_pg_buff_attr.size = attr[SIZE]
            else:
                attr_item_min.attr.ingress_port_pg_buff_attr.is_lossy = False
                attr_item_min.attr.ingress_port_pg_buff_attr.size = 0
                attr_item_min.attr.ingress_port_pg_buff_attr.xon = 0
                attr_item_min.attr.ingress_port_pg_buff_attr.xoff = 0
                
            print ("[%d] type  = SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E"% (i))
            print ("[%d] size  = %d" % (i, attr_item_min.attr.ingress_port_pg_buff_attr.size) )
            print ("[%d] PG    = %d" % (i, attr_item_min.attr.ingress_port_pg_buff_attr.pg) )
            print ("[%d] is_lossy (0=Lossless)  = %d" % (i, attr_item_min.attr.ingress_port_pg_buff_attr.is_lossy) )
            print ("[%d] Xon  = %d" % (i, attr_item_min.attr.ingress_port_pg_buff_attr.xon) )
            print ("[%d] Xoff  = %d" % (i, attr_item_min.attr.ingress_port_pg_buff_attr.xoff) )
            if attr_item_min.attr.ingress_port_pg_buff_attr.is_lossy:
                print ("[%d] override_default  = %d" % (i, pipe_latency.override_default) )
                print ("[%d] latency  = %d" % (i, pipe_latency.size) )
            
        elif attr[TYPE]==SX_COS_INGRESS_PORT_ATTR_E:
            attr_item_min.type = SX_COS_INGRESS_PORT_ATTR_E
            attr_item_min.attr.ingress_port_buff_attr.pool_id = attr[POOL_ID]
                
            if cmd == SX_ACCESS_CMD_SET:
                attr_item_min.attr.ingress_port_buff_attr.size = attr[SIZE]
            else:
                attr_item_min.attr.ingress_port_buff_attr.size = 0
                
            print ("[%d] type  = SX_COS_INGRESS_PORT_ATTR_E"% (i))
            print ("[%d] size  = %d" % (i, attr_item_min.attr.ingress_port_buff_attr.size) )
            print ("[%d] Pool id    = %d" % (i, attr_item_min.attr.ingress_port_buff_attr.pool_id) )
        
        elif attr[TYPE]==SX_COS_EGRESS_PORT_ATTR_E:
            attr_item_min.type = SX_COS_EGRESS_PORT_ATTR_E
            attr_item_min.attr.egress_port_buff_attr.pool_id = attr[POOL_ID]
                
            if cmd == SX_ACCESS_CMD_SET:
                attr_item_min.attr.egress_port_buff_attr.size = attr[SIZE]
            else:
                attr_item_min.attr.egress_port_buff_attr.size = 0
                
            print ("[%d] type  = SX_COS_EGRESS_PORT_ATTR_E"% (i))
            print ("[%d] size  = %d" % (i, attr_item_min.attr.egress_port_buff_attr.size) )
            print ("[%d] Pool id    = %d" % (i, attr_item_min.attr.egress_port_buff_attr.pool_id) )    
        
        elif attr[TYPE]==SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
            attr_item_min.type = SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E
            attr_item_min.attr.egress_port_tc_buff_attr.pool_id = attr[POOL_ID]
            attr_item_min.attr.egress_port_tc_buff_attr.tc = attr[TC]
                
            if cmd == SX_ACCESS_CMD_SET:
                attr_item_min.attr.egress_port_tc_buff_attr.size = attr[SIZE]
            else:
                attr_item_min.attr.egress_port_tc_buff_attr.size = 0
                
            print ("[%d] type  = SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E"% (i))
            print ("[%d] size  = %d" % (i, attr_item_min.attr.egress_port_tc_buff_attr.size) )
            print ("[%d] TC    = %d" % (i, attr_item_min.attr.egress_port_tc_buff_attr.tc) )   
            print ("[%d] Pool id    = %d" % (i, attr_item_min.attr.egress_port_tc_buff_attr.pool_id) )    
        
        elif attr[TYPE]==SX_COS_MULTICAST_ATTR_E:
            attr_item_min.type = SX_COS_MULTICAST_ATTR_E
            attr_item_min.attr.multicast_buff_attr.pool_id = attr[POOL_ID]
            attr_item_min.attr.multicast_buff_attr.sp = attr[SP]
                
            if cmd == SX_ACCESS_CMD_SET:
                attr_item_min.attr.multicast_buff_attr.size = attr[SIZE]
            else:
                attr_item_min.attr.multicast_buff_attr.size = 0
                
            print ("[%d] type  = SX_COS_MULTICAST_ATTR_E"% (i))
            print ("[%d] size  = %d" % (i, attr_item_min.attr.multicast_buff_attr.size) )
            print ("[%d] SP    = %d" % (i, attr_item_min.attr.multicast_buff_attr.sp) ) 
            print ("[%d] Pool id    = %d" % (i, attr_item_min.attr.multicast_buff_attr.pool_id) ) 
        
        print ("\n") 
                                                                    
        attr_item_min = sx_cos_port_buffer_attr_t_arr_setitem(port_buffer_attr_list_p, i, attr_item_min)
        
        i = i + 1
        
    rc = sx_api_cos_port_buff_type_set(handle, cmd, log_port, port_buffer_attr_list_p, port_buffer_attr_cnt)
    print ("sx_api_cos_port_buff_type_set [cmd=%d, log_port=0x%x , cnt=%d, rc=%d] " % ( cmd, log_port, port_buffer_attr_cnt, rc) )
    if rc != 0:
        sys.exit(errno.EACCES)   

def cos_sb_buff_get(attr_params_list):
    port_buffer_attr_cnt = new_uint32_t_p()
    port_buffer_attr_list_p = new_sx_cos_port_buffer_attr_t_arr(len(attr_params_list))
    uint32_t_p_assign(port_buffer_attr_cnt, len(attr_params_list))

    i = 0
    """ Set type and relevant parameters """
    for attr in attr_params_list:        
        attr_item_min = sx_cos_port_buffer_attr_t_arr_getitem(port_buffer_attr_list_p, i)
        if attr[TYPE]==SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
            attr_item_min.type = SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E
            attr_item_min.attr.ingress_port_pg_buff_attr.pg = attr[PG]
            attr_item_min.attr.ingress_port_pg_buff_attr.pool_id = attr[POOL_ID]
            
        elif attr[TYPE]==SX_COS_INGRESS_PORT_ATTR_E:
            attr_item_min.type = SX_COS_INGRESS_PORT_ATTR_E
            attr_item_min.attr.ingress_port_buff_attr.pool_id = attr[POOL_ID]
        
        elif attr[TYPE]==SX_COS_EGRESS_PORT_ATTR_E:
            attr_item_min.type = SX_COS_EGRESS_PORT_ATTR_E
            attr_item_min.attr.egress_port_buff_attr.pool_id = attr[POOL_ID]  
        
        elif attr[TYPE]==SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
            attr_item_min.type = SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E
            attr_item_min.attr.egress_port_tc_buff_attr.pool_id = attr[POOL_ID]
            attr_item_min.attr.egress_port_tc_buff_attr.tc = attr[TC]
        
        elif attr[TYPE]==SX_COS_MULTICAST_ATTR_E:
            attr_item_min.type = SX_COS_MULTICAST_ATTR_E
            attr_item_min.attr.multicast_buff_attr.pool_id = attr[POOL_ID]
            attr_item_min.attr.multicast_buff_attr.sp = attr[SP]
                                                                    
        attr_item_min = sx_cos_port_buffer_attr_t_arr_setitem(port_buffer_attr_list_p, i, attr_item_min)
        
        i = i + 1
        
    rc = sx_api_cos_port_buff_type_get(handle, log_port, port_buffer_attr_list_p, port_buffer_attr_cnt)
    if rc != 0:
        sys.exit(errno.EACCES)

    """ Print list """
    attr_cnt = uint32_t_p_value(port_buffer_attr_cnt)
    print ("sx_api_cos_port_buff_type_get [log_port=0x%x , cnt=%d, rc=%d] " % ( log_port, attr_cnt, rc) )
    print ("--------------------------------------------------------------------")
    print ("Respond count:  %d" % ( attr_cnt) )
    if len(attr_params_list) > 0:
        for i in range(0,attr_cnt):
            port_buffer_attr_item = sx_cos_port_buffer_attr_t_arr_getitem(port_buffer_attr_list_p, i)
            
            if port_buffer_attr_item.type==SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
                pipe_latency = port_buffer_attr_item.attr.ingress_port_pg_buff_attr.pipeline_latency
                print ("[%d] type  = SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E"% (i))
                print ("[%d] size  = %d" % (i, port_buffer_attr_item.attr.ingress_port_pg_buff_attr.size) )
                print ("[%d] PG    = %d" % (i, port_buffer_attr_item.attr.ingress_port_pg_buff_attr.pg) )
                print ("[%d] is_lossy (0=Lossless)  = %d" % (i, port_buffer_attr_item.attr.ingress_port_pg_buff_attr.is_lossy) )
                print ("[%d] Xon  = %d" % (i, port_buffer_attr_item.attr.ingress_port_pg_buff_attr.xon) )
                print ("[%d] Xoff  = %d" % (i, port_buffer_attr_item.attr.ingress_port_pg_buff_attr.xoff) )
                print ("[%d] override_default  = %d" % (i, pipe_latency.override_default) )
                print ("[%d] latency  = %d" % (i, pipe_latency.size) )
            elif port_buffer_attr_item.type==SX_COS_INGRESS_PORT_ATTR_E:
                print ("[%d] type  = SX_COS_INGRESS_PORT_ATTR_E"% (i))
                print ("[%d] size  = %d" % (i, port_buffer_attr_item.attr.ingress_port_buff_attr.size) )
                print ("[%d] Pool id    = %d" % (i, port_buffer_attr_item.attr.ingress_port_buff_attr.pool_id) )
            elif port_buffer_attr_item.type==SX_COS_EGRESS_PORT_ATTR_E:
                print ("[%d] type  = SX_COS_EGRESS_PORT_ATTR_E"% (i))
                print ("[%d] size  = %d" % (i, port_buffer_attr_item.attr.egress_port_buff_attr.size) )
                print ("[%d] Pool id    = %d" % (i, port_buffer_attr_item.attr.egress_port_buff_attr.pool_id) )
            elif port_buffer_attr_item.type==SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
                print ("[%d] type  = SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E"% (i))
                print ("[%d] size  = %d" % (i, port_buffer_attr_item.attr.egress_port_tc_buff_attr.size) )
                print ("[%d] TC    = %d" % (i, port_buffer_attr_item.attr.egress_port_tc_buff_attr.tc) )   
                print ("[%d] Pool id    = %d" % (i, port_buffer_attr_item.attr.egress_port_tc_buff_attr.pool_id) ) 
            elif port_buffer_attr_item.type==SX_COS_MULTICAST_ATTR_E:
                print ("[%d] type  = SX_COS_MULTICAST_ATTR_E"% (i))
                print ("[%d] size  = %d" % (i, port_buffer_attr_item.attr.multicast_buff_attr.size) )
                print ("[%d] SP    = %d" % (i, port_buffer_attr_item.attr.multicast_buff_attr.sp) ) 
                print ("[%d] Pool id    = %d" % (i, port_buffer_attr_item.attr.multicast_buff_attr.pool_id) ) 
            print ("\n")      

######################################################
#    main
######################################################
def main():

    get_buff_stat()
    
    #default ingress/egress pools
    ing_pool_id = 0
    egr_pool_id = 10
    
    """ PORT TYPE BUFF CMD: SET """
    print ("\n\n---------------  PORT TYPE BUFF SET - CMD: SET -------------------")
    params_list = []
    params_list.append({TYPE:          SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                        IS_LOSSY:      True,
                        PG:            1,
                        POOL_ID:       ing_pool_id,
                        OVERRIDE_DEFAULT: 0,
                        SIZE:          20})
    params_list.append({TYPE:          SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                        IS_LOSSY:      True,
                        PG:            2,
                        POOL_ID:       ing_pool_id,
                        OVERRIDE_DEFAULT: 1,
                        LATENCY:       30,
                        SIZE:          20})
    params_list.append({TYPE:          SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                        IS_LOSSY:      True,
                        PG:            3,
                        POOL_ID:       ing_pool_id,
                        OVERRIDE_DEFAULT: 0,
                        SIZE:          20})
    params_list.append({TYPE:      SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E,
                        TC:        5,
                        POOL_ID:   egr_pool_id,
                        SIZE:      15})
    params_list.append({TYPE:      SX_COS_INGRESS_PORT_ATTR_E,
                        POOL_ID:   ing_pool_id,
                        SIZE:      15})
    params_list.append({TYPE:      SX_COS_EGRESS_PORT_ATTR_E,
                        POOL_ID:   egr_pool_id,
                        SIZE:      15})
    """params_list.append({TYPE:      SX_COS_MULTICAST_ATTR_E,
                        SP:        5,
                        POOL_ID:   4,
                        SIZE:      15})"""
    cos_sb_buff_set(SX_ACCESS_CMD_SET, params_list)

    get_buff_stat()
    
    """ PORT TYPE BUFF GET """
    print ("\n\n---------------  PORT TYPE BUFF GET ------------------------------")
    params_list = []
    params_list.append({TYPE:      SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                        POOL_ID:   ing_pool_id,
                        PG:        1})
    params_list.append({TYPE:      SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                        POOL_ID:   ing_pool_id,
                        PG:        2})
    params_list.append({TYPE:      SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                        POOL_ID:   ing_pool_id,
                        PG:        3})
    params_list.append({TYPE:      SX_COS_INGRESS_PORT_ATTR_E,
                        POOL_ID:   ing_pool_id})
    params_list.append({TYPE:      SX_COS_EGRESS_PORT_ATTR_E,
                        POOL_ID:   egr_pool_id})
    params_list.append({TYPE:      SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E,
                        TC:        0,
                        POOL_ID:   egr_pool_id})
    cos_sb_buff_get(params_list)

    """ PORT TYPE BUFF CMD: DELETE """
    print ("\n\n---------------  PORT TYPE BUFF SET - CMD: DELETE ----------------")
    params_list = []
    params_list.append({TYPE:      SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                        POOL_ID:   ing_pool_id,
                        PG:        1})
    params_list.append({TYPE:      SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                        POOL_ID:   ing_pool_id,
                        PG:        2})
    cos_sb_buff_set(SX_ACCESS_CMD_DELETE, params_list)

    get_buff_stat()
    
    """ PORT TYPE BUFF GET """
    print ("\n\n---------------  PORT TYPE BUFF GET ------------------------------")
    params_list = []
    params_list.append({TYPE:      SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                        POOL_ID:   ing_pool_id,
                        PG:        1})
    params_list.append({TYPE:      SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                        POOL_ID:   ing_pool_id,
                        PG:        2})
    cos_sb_buff_get(params_list)
    
    """ PORT TYPE BUFF CMD: DELETE_ALL """
    print ("\n\n---------------  PORT TYPE BUFF SET - CMD: DELETE_ALL ------------")
    params_list = []
    cos_sb_buff_set(SX_ACCESS_CMD_DELETE_ALL, params_list)
    
    """ PORT TYPE BUFF GET """
    print ("\n\n---------------  PORT TYPE BUFF GET ------------------------------")
    params_list = []
    params_list.append({TYPE:      SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                        POOL_ID:   ing_pool_id,
                        PG:        1})
    params_list.append({TYPE:      SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                        POOL_ID:   ing_pool_id,
                        PG:        2})
    params_list.append({TYPE:      SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                        POOL_ID:   ing_pool_id,
                        PG:        3})
    params_list.append({TYPE:      SX_COS_INGRESS_PORT_ATTR_E,
                        POOL_ID:   ing_pool_id})
    params_list.append({TYPE:      SX_COS_EGRESS_PORT_ATTR_E,
                        POOL_ID:   egr_pool_id})
    params_list.append({TYPE:      SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E,
                        TC:        5,
                        POOL_ID:   egr_pool_id})
    cos_sb_buff_get(params_list)
        
if __name__ == "__main__":
    main()      
    
