#!/usr/bin/env python
'''
This file contains Python command example for the FDB DUMP module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.
'''
import sys, errno
import os
from python_sdk_api.sx_api import *
from pprint import pprint
from test_infra_common import *

entry_dict = {0 : 'STATIC', 1 : 'REMOTE', 2 : 'NONAGEABLE', 3 : 'AGEABLE', 4 : 'ALL', \
               5 : 'AGED', 6 : 'NONAGED', 7 : 'POLICY_INVALID'}
action_dict = {0 : 'FORWARD', 1 : 'MIRROR_TO_CPU', 2 : 'TRAP', 3 : 'FORWARD_TO_ROUTER', \
               4 : 'INVALID', 5 : 'INVALID', 6 : 'INVALID', 7 : 'INVALID', 8 : 'INVALID', 9 : 'INVALID', 10 : 'INVALID', 11 : 'INVALID', \
               12 : 'INVALID', 13 : 'INVALID', 14 : 'INVALID', 15 : 'DISCARD'}
dest_type_dict = {0: 'Logical Port', 1: 'Next Hop'}
ERR_FILE_LOCATION = '/tmp/python_err_log.txt'

file_exist = os.path.isfile(ERR_FILE_LOCATION)
sys.stderr = open(ERR_FILE_LOCATION, 'w')
if not file_exist:
    os.chmod(ERR_FILE_LOCATION, 0o777);
    
old_stdout = redirect_stdout()
rc, handle = sx_api_open(None)
sys.stdout = os.fdopen(old_stdout, 'w')
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

swid_cnt_p = new_uint32_t_p()
uint32_t_p_assign(swid_cnt_p, 0)
rc = sx_api_port_swid_list_get(handle, None, swid_cnt_p)
if rc != SX_STATUS_SUCCESS:
    print "sx_api_port_swid_list_get failed, rc = %d" % (rc)
    exit()

swid_cnt = uint32_t_p_value(swid_cnt_p)
swid_list_p = new_sx_swid_t_arr(swid_cnt)
rc = sx_api_port_swid_list_get(handle, swid_list_p, swid_cnt_p)
if rc != SX_STATUS_SUCCESS:
    print "sx_api_port_swid_list_get failed, rc = %d" % (rc)
    exit()
swid_cnt = uint32_t_p_value(swid_cnt_p)

mac_entry_arr = new_sx_fdb_uc_mac_addr_params_t_arr(64)
key_p = new_sx_fdb_uc_mac_addr_params_t_p()
key_filter_p = new_sx_fdb_uc_key_filter_t_p()
data_cnt_p = new_uint32_t_p()
uint32_t_p_assign(data_cnt_p,64)
header_printed = False

for i in range(0, swid_cnt):
    swid = sx_swid_t_arr_getitem(swid_list_p, i)

    rc = sx_api_fdb_uc_mac_addr_get(handle, swid, SX_ACCESS_CMD_GET_FIRST, SX_FDB_UC_ALL, key_p, key_filter_p, mac_entry_arr, data_cnt_p)
    if (rc != SX_STATUS_SUCCESS):
        print "sx_api_fdb_uc_mac_addr_get failed, rc = %d" % (rc)
        exit()
    data_cnt = uint32_t_p_value(data_cnt_p)

    read_number = 0
    while (data_cnt == 64):
        for i in range(0, data_cnt):
            if not header_printed:
                print "------------------------------------------------------------------------------------------------------------------"
                print "|%5s|%10s|%10s|%20s|%10s|%20s|%10s|%20s|" % ("SWID", "Entry", "VID", "MAC", "Port", "Action", "Entry Type", "Destination Type")
                print "------------------------------------------------------------------------------------------------------------------"
                header_printed = True		

            mac_entry = sx_fdb_uc_mac_addr_params_t_arr_getitem(mac_entry_arr, i)
            log_port = "0x%x" % mac_entry.log_port
            print "|%5d|%10d|%10d|%20s|%10s|%20s|%10s|%20s|" % (swid, read_number*64 + i, mac_entry.fid_vid, mac_entry.mac_addr.to_str(), 
	                                                        log_port, action_dict[mac_entry.action], entry_dict[mac_entry.entry_type], dest_type_dict[mac_entry.dest_type])
            print "-----------------------------------------------------------------------------------------------------------------"
 
        key_p.fid_vid = mac_entry.fid_vid
        key_p.mac_addr = mac_entry.mac_addr
        rc = sx_api_fdb_uc_mac_addr_get(handle, swid, SX_ACCESS_CMD_GETNEXT, SX_FDB_UC_ALL, key_p, key_filter_p, mac_entry_arr, data_cnt_p)
        if (rc != SX_STATUS_SUCCESS):
            print "sx_api_fdb_uc_mac_addr_get failed, rc = %d" % (rc)
            exit()

        data_cnt = uint32_t_p_value(data_cnt_p)
        read_number = read_number + 1

    for i in range(0, data_cnt):
        if not header_printed:
            print "------------------------------------------------------------------------------------------------------------------"
            print "|%5s|%10s|%10s|%20s|%10s|%20s|%10s|%20s|" % ("SWID", "Entry", "VID", "MAC", "Port", "Action", "Entry Type", "Destination Type")
            print "------------------------------------------------------------------------------------------------------------------"
            header_printed = True
	    
        mac_entry = sx_fdb_uc_mac_addr_params_t_arr_getitem(mac_entry_arr, i)
        log_port = "0x%x" % mac_entry.log_port
        print "|%5d|%10d|%10d|%20s|%10s|%20s|%10s|%20s|" % (swid, read_number*64 + i, mac_entry.fid_vid, mac_entry.mac_addr.to_str(),
	                                                    log_port, action_dict[mac_entry.action], entry_dict[mac_entry.entry_type], dest_type_dict[mac_entry.dest_type])
        print "------------------------------------------------------------------------------------------------------------------"









