#!/usr/bin/env python
"""
This file contains Python command monitoring firware degug setting get/set.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

 
"""
import sys, random
import traceback, errno
import os
import inspect

from python_sdk_api.sx_api import *

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

vlan_id = random.randint(1, 100)

# set multicast flood  mode 
urmc_flood_mode = random.choice([SX_FDB_UNREG_MC_FLOOD, SX_FDB_UNREG_MC_PRUNE])
rc = sx_api_fdb_unreg_mc_flood_mode_set(handle, 0, vlan_id, urmc_flood_mode)
print("sx_api_fdb_unreg_mc_flood_mode_set swid 0  vlan id: %d, mode: %d, rc: %d " %(vlan_id, urmc_flood_mode, rc))

# get multicast flood  mode 
sx_fdb_unreg_flood_mode_t_ptr = new_sx_fdb_unreg_flood_mode_t_p()
rc = sx_api_fdb_unreg_mc_flood_mode_get(handle, 0, vlan_id, sx_fdb_unreg_flood_mode_t_ptr)
urmc_flood_mode = sx_fdb_unreg_flood_mode_t_p_value(sx_fdb_unreg_flood_mode_t_ptr)
print("sx_api_fdb_unreg_mc_flood_mode_get swid 0  vlan id: %d, mode: %d, rc: %d " %(vlan_id, urmc_flood_mode, rc))

# set multicast flood ports for vlan
total_dev_ports = 64
port_attributes_list = new_sx_port_attributes_t_arr(total_dev_ports)
dev_portcount_p = new_uint32_t_p()
uint32_t_p_assign(dev_portcount_p, total_dev_ports)
rc = sx_api_port_device_get(handle, 1 , 0, port_attributes_list,  dev_portcount_p)

port_cnt = 2
set_portlist = []
port_list = new_sx_port_log_id_t_arr(port_cnt)
log_port = sx_port_attributes_t_arr_getitem(port_attributes_list, 0).log_port
sx_port_log_id_t_arr_setitem(port_list, 0, log_port)
set_portlist.append(log_port)
log_port = sx_port_attributes_t_arr_getitem(port_attributes_list, 1).log_port
sx_port_log_id_t_arr_setitem(port_list, 1, log_port)
set_portlist.append(log_port)
rc = sx_api_fdb_unreg_mc_flood_ports_set(handle, 0, vlan_id, port_list, port_cnt)
print("sx_api_fdb_unreg_mc_flood_ports_set swid 0  vlan id: %d, rc: %d " %(vlan_id, rc))


# get multicast flood ports count
portcount_p = new_uint32_t_p()
uint32_t_p_assign(portcount_p, 0)
rc = sx_api_fdb_unreg_mc_flood_ports_get(handle, 0, vlan_id, None, portcount_p)
portcount = uint32_t_p_value(portcount_p)
print("sx_api_fdb_unreg_mc_flood_ports_get swid 0  vlan id: %d, port count=%d, rc: %d " %(vlan_id, portcount, rc))

#get port list
get_portlist = []
portlist_arr = new_sx_port_log_id_t_arr(portcount)
rc = sx_api_fdb_unreg_mc_flood_ports_get(handle, 0, vlan_id, portlist_arr, portcount_p)
PORT1_GET = sx_port_log_id_t_arr_getitem(portlist_arr, 0)
get_portlist.append(PORT1_GET)
PORT2_GET = sx_port_log_id_t_arr_getitem(portlist_arr, 1)
get_portlist.append(PORT2_GET)
assert  sorted(set_portlist) == sorted(get_portlist), "port list mismatch"
print("sx_api_fdb_unreg_mc_flood_ports_get swid 0  vlan id: %d, port1=0x%x, port2=0x%x rc: %d " %(vlan_id, PORT1_GET, PORT2_GET, rc))

