#!/usr/bin/env python
# -*- python -*-

import sys
import os
import Queue
import ctypes
import struct
import socket
sys.path.append(os.path.dirname(os.path.realpath(__file__)) + '/../test_infra/')
sys.path.append(os.path.dirname(os.path.realpath(__file__)) + '/../test_infra/sdk_common_infra')
sys.path.append('/rdmzsysgwork/mswg/projects/sx_sdk/sx_fit_regression/libs/tools/')
from common_infra_acl import *
from test_infra_common import *

from python_sdk_api.sx_api import *

def main():

    print "NOTE: ACL ranges are supported by Spectrum2 only."
    rc, handle = sx_api_open(None)
    if (rc != SX_STATUS_SUCCESS):
        print "Failed to open api handle.\nPlease check that SDK is running."
        sys.exit(errno.EACCES)

    port_list = mapPortAndInterfaces(handle)
    ingress_log_port = port_list[0]

    # Create the ACL range (L4 Port range)
    range_entry = sx_acl_range_entry_t()
    range_entry.ip_version = SX_IP_VERSION_IPV4_IPV6
    range_entry.direction = SX_ACL_PORT_DIRECTION_BOTH
    range_entry.l4_protocol = SX_ACL_L4_TYPE_TCP_UDP
    range_entry.outer_inner = SX_ACL_PORT_RANGE_IP_HEADER_INNER
    range_entry.range_type =  SX_ACL_RANGE_TYPE_L4_PORT_E
    range_entry.range_limits = sx_acl_range_limits_t()
    range_entry.range_limits.min = 1000
    range_entry.range_limits.max = 2000
    
    port_range_id_p = new_sx_acl_port_range_id_t_p()
    
    rc = sx_api_acl_range_set(handle,
                              SX_ACCESS_CMD_ADD,
                              range_entry,
                              port_range_id_p)
    check_sdk_rc(rc)
    port_range_id = sx_acl_port_range_id_t_p_value(port_range_id_p)

    keys_list = new_sx_acl_key_t_arr(1)
    sx_acl_key_t_arr_setitem(keys_list, 0, FLEX_ACL_KEY_L4_PORT_RANGE)

    key_handle_p = new_sx_acl_key_type_t_p()
    acl_key_create_delete(handle, SX_ACCESS_CMD_CREATE, key_handle_p, [sx_acl_key_t_arr_getitem(keys_list, 0)])
    key_handle = sx_acl_key_type_t_p_value(key_handle_p)

    rules_list = new_sx_flex_acl_flex_rule_t_arr(100)
    rules = []
    acl_rules_init(rules, rules_list, 100, key_handle, 20)

    region_id_p = new_sx_acl_region_id_t_p()
    acl_region_handle(handle, SX_ACCESS_CMD_CREATE, key_handle, 10, region_id_p)
    region_id = sx_acl_region_id_t_p_value(region_id_p)

    dir = SX_ACL_DIRECTION_INGRESS
    acl_id_p = new_sx_acl_id_t_p()
    acl_handle(handle, SX_ACCESS_CMD_CREATE, dir, acl_id_p, region_id)
    acl_id = sx_acl_id_t_p_value(acl_id_p)

    group_id_p = new_sx_acl_id_t_p()
    acl_group_handle(handle, SX_ACCESS_CMD_CREATE, dir, group_id_p, [acl_id])
    group_id = sx_acl_id_t_p_value(group_id_p)

    port_bind(handle, SX_ACCESS_CMD_BIND, ingress_log_port, group_id)
   
    offsets_list = new_sx_acl_rule_offset_t_arr(100)
    sx_acl_rule_offset_t_arr_setitem(offsets_list, 0, 0)
    rules[0].valid = 1
    rules[0].key_desc_count = 1
    rules[0].action_count = 1

    key_desc = sx_flex_acl_key_desc_t()
    key_desc.key_id = FLEX_ACL_KEY_L4_PORT_RANGE
    key_desc.key.l4_port_range.port_range_cnt = 1
    sx_acl_port_range_id_t_arr_setitem(key_desc.key.l4_port_range.port_range_list, 0, port_range_id)

    sx_flex_acl_key_desc_t_arr_setitem(rules[0].key_desc_list_p, 0, key_desc)
    key_desc.mask.l4_port_range = True

    sx_flex_acl_key_desc_t_arr_setitem(rules[0].key_desc_list_p, 0, key_desc)

    action = sx_flex_acl_flex_action_t()
    action.type = SX_FLEX_ACL_ACTION_SET_DSCP
    action.fields.action_set_dscp.dscp_val = 28
    sx_flex_acl_flex_action_t_arr_setitem(rules[0].action_list_p, 0, action)

    sx_flex_acl_flex_rule_t_arr_setitem(rules_list, 0, rules[0])
    rc = sx_api_acl_flex_rules_set(handle,
                                   SX_ACCESS_CMD_SET,
                                   region_id,
                                   offsets_list,
                                   rules_list,
                                   1)
    check_sdk_rc(rc)


    # Edit the ACL range (IP Length range)
    range_entry = sx_acl_range_entry_t()
    range_entry.ip_version = SX_IP_VERSION_IPV6
    range_entry.outer_inner = SX_ACL_PORT_RANGE_IP_HEADER_INNER
    range_entry.range_type = SX_ACL_RANGE_TYPE_IP_LENGTH_E
    range_entry.range_limits = sx_acl_range_limits_t()
    range_entry.range_limits.min = 100
    range_entry.range_limits.max = 3000

    rc = sx_api_acl_range_set(handle,
                              SX_ACCESS_CMD_EDIT,
                              range_entry,
                              port_range_id_p)
    check_sdk_rc(rc)


    # Edit the ACL range (TTL range)
    range_entry = sx_acl_range_entry_t()
    range_entry.ip_version = SX_IP_VERSION_IPV4
    range_entry.l4_protocol = SX_ACL_L4_TYPE_TCP
    range_entry.outer_inner = SX_ACL_PORT_RANGE_IP_HEADER_OUTER
    range_entry.range_type = SX_ACL_RANGE_TYPE_TTL_E
    range_entry.range_limits = sx_acl_range_limits_t()
    range_entry.range_limits.min = 50
    range_entry.range_limits.max = 160

    rc = sx_api_acl_range_set(handle,
                              SX_ACCESS_CMD_EDIT,
                              range_entry,
                              port_range_id_p)
    check_sdk_rc(rc)


    # Creating Custom Byte Set
    key_ids = new_sx_acl_key_t_arr(2)
    key_ids_count_p = new_uint32_t_p()
    uint32_t_p_assign(key_ids_count_p, 2)

    custom_bytes_set_attributes = sx_acl_custom_bytes_set_attributes_t()
    custom_bytes_set_attributes.extraction_point.extraction_group_type = SX_ACL_CUSTOM_BYTES_EXTRACTION_GROUP_L3
    custom_bytes_set_attributes.extraction_point.params.extraction_l3_group.extraction_ipv4.extraction_point_type = SX_ACL_CUSTOM_BYTES_EXTRACTION_POINT_TYPE_IPV4_START_OF_HEADER
    custom_bytes_set_attributes.extraction_point.params.extraction_l3_group.extraction_ipv4.offset = 28
        
    rc = sx_api_acl_custom_bytes_set(handle,
                                     SX_ACCESS_CMD_CREATE,
                                     custom_bytes_set_attributes,
                                     key_ids,
                                     key_ids_count_p)
    check_sdk_rc(rc)


    # Edit the ACL range (Custom Byte range)
    range_entry = sx_acl_range_entry_t()
    range_entry.ip_version = SX_IP_VERSION_IPV4
    range_entry.l4_protocol = SX_ACL_L4_TYPE_TCP
    range_entry.outer_inner = SX_ACL_PORT_RANGE_IP_HEADER_OUTER
    range_entry.range_type = SX_ACL_RANGE_TYPE_CUSTOM_BYTE_SET_E
    range_entry.range_limits = sx_acl_range_limits_t()
    range_entry.range_limits.min = 0x1111
    range_entry.range_limits.max = 0x8888
    range_entry.range_attr = sx_acl_range_attr_t()
    range_entry.range_attr.custom_bytes_range_attr = sx_custom_bytes_range_attr_t()
    range_entry.range_attr.custom_bytes_range_attr.custom_bytes_set_key_id = sx_acl_key_t_arr_getitem(key_ids, 0)

    rc = sx_api_acl_range_set(handle,
                              SX_ACCESS_CMD_EDIT,
                              range_entry,
                              port_range_id_p)
    check_sdk_rc(rc)

    # Edit the ACL range (UTC range)
    range_entry = sx_acl_range_entry_t()
    range_entry.ip_version = SX_IP_VERSION_IPV4_IPV6
    range_entry.l4_protocol = SX_ACL_L4_TYPE_TCP
    range_entry.outer_inner = SX_ACL_PORT_RANGE_IP_HEADER_OUTER
    range_entry.range_type = SX_ACL_RANGE_TYPE_UTC_29_14_E
    range_entry.range_limits = sx_acl_range_limits_t()
    range_entry.range_limits.min = 0x1111
    range_entry.range_limits.max = 0x3333

    rc = sx_api_acl_range_set(handle,
                              SX_ACCESS_CMD_EDIT,
                              range_entry,
                              port_range_id_p)
    check_sdk_rc(rc)

    #####################################
    # Clean up
    #####################################

    port_bind(handle, SX_ACCESS_CMD_UNBIND, ingress_log_port, group_id)

    sx_acl_rule_offset_t_arr_setitem(offsets_list, 0, 0)
    rc = sx_api_acl_flex_rules_set(handle,
                                   SX_ACCESS_CMD_DELETE,
                                   region_id,
                                   offsets_list,
                                   rules_list,
                                   1)
    check_sdk_rc(rc)

    acl_group_handle(handle, SX_ACCESS_CMD_DESTROY, dir, group_id_p, [])
    acl_handle(handle, SX_ACCESS_CMD_DESTROY, dir, acl_id_p, region_id)
    acl_region_handle(handle, SX_ACCESS_CMD_DESTROY, key_handle, 10, region_id_p)
    acl_key_create_delete(handle, SX_ACCESS_CMD_DELETE, key_handle_p, [sx_acl_key_t_arr_getitem(keys_list, 0)])

    rc = sx_api_acl_custom_bytes_set(handle,
                                     SX_ACCESS_CMD_DESTROY,
                                     custom_bytes_set_attributes,
                                     key_ids,
                                     key_ids_count_p)
    check_sdk_rc(rc)

    rc = sx_api_acl_range_set(handle,
                              SX_ACCESS_CMD_DELETE,
                              range_entry,
                              port_range_id_p)
    check_sdk_rc(rc)

if __name__ == "__main__":
        main()

