#!/usr/bin/env python
'''
This file contains Python command example for the LAG iterator get function.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

need to add description 
'''
import os
import sys
import errno
import random
import time
import traceback

from python_sdk_api.sx_api import *
print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)
random.seed()
        
###################################################### 
#    defines
######################################################
MAX_KEY_NUM = 32
TEST_SUCCESS = 0
TEST_FAILED = 1

######################################################
#    main
######################################################
def main():
     
    try:
        ret_code = TEST_SUCCESS       
        key_list = []

        lag_id_p = new_sx_port_log_id_t_p()
        # Create some LAG IDs
        for i in xrange(1, MAX_KEY_NUM + 1):
            rc = sx_api_lag_port_group_set(handle, SX_ACCESS_CMD_CREATE, 0, lag_id_p, None ,0)
            if rc != SX_STATUS_SUCCESS:
                raise Exception("sx_api_lag_port_group_set failed, rc=%d" % rc)
            lag_id = sx_port_log_id_t_p_value(lag_id_p)
            key_list.append(lag_id)
            
        key_list.sort()
        num_keys = len(key_list)
        print "Created LAG ID list: (%d keys)" % num_keys
        for x in key_list:
            print "LAG ID: 0x%x" % x
            
        print "--------------------------------GET---------------------------------------------------------------------------"
    
        key_cnt_p = new_uint32_t_p()
        key_list_p = new_sx_port_log_id_t_arr(MAX_KEY_NUM)
        # Use GET to get all elements in the list
        key_get_list=[]
        for key in key_list:
            uint32_t_p_assign(key_cnt_p, 1)
            print "GET    lag_id: 0x%x, key_cnt: %d" % (key, 1)
            rc = sx_api_lag_port_group_iter_get(handle, SX_ACCESS_CMD_GET, 0, key, None, key_list_p, key_cnt_p)
            if rc != SX_STATUS_SUCCESS:
                raise Exception("sx_api_lag_port_group_iter_get failed, rc=%d" % rc)
            
            key_cnt = uint32_t_p_value(key_cnt_p)
            assert key_cnt == 1 , "Returned key count is not 1"
                
            key_get = sx_port_log_id_t_arr_getitem(key_list_p, 0)
            print "Return lag_id: 0x%x, key_cnt: %d" % (key_get, key_cnt) 
            
            
        print "--------------------------------COUNT---------------------------------------------------------------------------"        
            
        # Get Count 
        uint32_t_p_assign(key_cnt_p, 0)

        rc = sx_api_lag_port_group_iter_get(handle, SX_ACCESS_CMD_GET, 0, 0, None, None, key_cnt_p)
        if rc != SX_STATUS_SUCCESS:
            raise Exception("sx_api_lag_port_group_iter_get failed, rc=%d" % rc)
        key_cnt = uint32_t_p_value(key_cnt_p)
        
        assert key_cnt == num_keys, "Count failed, expected %d, got %d" %(num_keys, key_cnt)
        print "Count successful, expected %d, got %d" %(num_keys, key_cnt)
        
            
        print "--------------------------------GET_FIRST---------------------------------------------------------------------------"
           
        num_get = random.randint(1, MAX_KEY_NUM) 
        uint32_t_p_assign(key_cnt_p, num_get)
        print "GET FIRST, requested count = %d, total count = %d" % (num_get, num_keys)
        rc = sx_api_lag_port_group_iter_get(handle, SX_ACCESS_CMD_GET_FIRST, 0, 0,  None, key_list_p, key_cnt_p)
        if rc != SX_STATUS_SUCCESS:
            raise Exception("sx_api_lag_port_group_iter_get failed, rc=%d" % rc)
        key_cnt = uint32_t_p_value(key_cnt_p)
       
        print "GET FIRST, returned count = %d" % key_cnt
        for i in range(0, key_cnt):
            key_get = sx_port_log_id_t_arr_getitem(key_list_p, i)
            key = key_list[i]
            assert key == key_get, "The content returned by GET FIRST is incorrect"
            print "lag_id: 0x%x" % (key_get)
            
            
        print "--------------------------------GET_NEXT---------------------------------------------------------------------------"
    
        num_get = random.randint(1, MAX_KEY_NUM) 
        uint32_t_p_assign(key_cnt_p, num_get)
        index = random.randint(0, num_keys - 1)
        key = key_list[index]
        print "GET NEXT %d keys after %dth key (lag_id: 0x%x), total count = %d" % (num_get, index+1, key, num_keys)
        rc = sx_api_lag_port_group_iter_get(handle, SX_ACCESS_CMD_GETNEXT, 0, key, None, key_list_p, key_cnt_p)
        if rc != SX_STATUS_SUCCESS:
            raise Exception("sx_api_lag_port_group_iter_get failed, rc=%d" % rc)
        key_cnt = uint32_t_p_value(key_cnt_p)
        print "GET NEXT %d keys after %dth key (lag_id: 0x%x), returned %d keys" % (num_get, index+1, key, key_cnt) 
        for i in range(0, key_cnt):
            key_get = sx_port_log_id_t_arr_getitem(key_list_p, i)
            key = key_list[index+i+1]
            assert key_get == key, "The content returned by GET NEXT is incorrect"
            print "lag_id: 0x%x" % (key_get)
            
            
        print "--------------------------------GET_NEXT with a non-existent key------------------------------------------------------"
        num_get = random.randint(1, MAX_KEY_NUM) 
        uint32_t_p_assign(key_cnt_p, num_get)
        index = random.randint(0, num_keys - 1)
        key = key_list[index]
        sx_port_log_id_t_p_assign(lag_id_p, key)
        rc = sx_api_lag_port_group_set(handle, SX_ACCESS_CMD_DESTROY, 0, lag_id_p, None, 0)
        if rc != SX_STATUS_SUCCESS:
            raise Exception("sx_api_lag_port_group_set failed, rc=%d" % rc)
        print "GET NEXT %d keys after key (lag_id: 0x%x), total count = %d" % (num_get, key, num_keys)

        rc = sx_api_lag_port_group_iter_get(handle, SX_ACCESS_CMD_GETNEXT, 0, key, None, key_list_p, key_cnt_p)
        if rc != SX_STATUS_SUCCESS:
            raise Exception("sx_api_lag_port_group_iter_get failed, rc=%d" % rc)
        key_cnt = uint32_t_p_value(key_cnt_p)
        print "GET NEXT %d keys after key (lag_id: 0x%x), returned %d keys" % (num_get, key, key_cnt) 
        for i in range(0, key_cnt):
            key_get = sx_port_log_id_t_arr_getitem(key_list_p, i)
            key = key_list[index+i+1]
            assert key_get == key, "The content returned by GET NEXT is incorrect"
            print "lag_id: 0x%x" % (key_get)
            
        key_list.remove(key_list[index])
         
        # Corner cases
                
        # Call GET with count > 1
        case = "Test case: Call GET with count > 1"
        print case 
        key_cnt = random.randint(2, MAX_KEY_NUM)
        uint32_t_p_assign(key_cnt_p, key_cnt)
        key = key_list[random.randint(0, num_keys - 1)]
        rc = sx_api_lag_port_group_iter_get(handle, SX_ACCESS_CMD_GET, 0, key, None, key_list_p, key_cnt_p)
        if rc != SX_STATUS_SUCCESS:
            raise Exception("sx_api_lag_port_group_iter_get failed, rc=%d in case %s" % (rc, case))
        key_cnt = uint32_t_p_value(key_cnt_p)
        assert key_cnt <= 1 , "Data count not 0 or 1 as expected  in case %s" % (case)            
        print case + " passed\n"       
            
        print "Cleaning up LAG IDs....\n"
        for key in key_list:
            sx_port_log_id_t_p_assign(lag_id_p, key)
            rc = sx_api_lag_port_group_set(handle, SX_ACCESS_CMD_DESTROY, 0, lag_id_p, None, 0)
            if rc != SX_STATUS_SUCCESS:
                raise Exception("sx_api_lag_port_group_set failed, rc=%d" % rc)
        
    except Exception, err:
        ret_code += TEST_FAILED
        print 'Exception of type %s occurred:\n%s' % (str(type(err)), str(err))
        traceback.print_exc()
        for key in key_list:
            sx_port_log_id_t_p_assign(lag_id_p, key)
            rc = sx_api_lag_port_group_set(handle, SX_ACCESS_CMD_DESTROY, 0, lag_id_p, None, 0)
            if rc != SX_STATUS_SUCCESS:
                raise Exception("sx_api_lag_port_group_set failed, rc=%d" % rc)
           
        
        
    finally:
        if ret_code:
            print "Test failed: Return code = %d" % ret_code
        else:
            print "All tests passed"
        

if __name__ == "__main__":
    main()



""" ############################################################################################ """
sx_api_close(handle)
""" ############################################################################################ """
