#!/usr/bin/env python
'''
This file contains Python command example for the PORT DUMP module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.
'''

oper_dict = {0 : 'NONE', 1 : 'UP', 2 : 'DOWN', 4 : 'DOWN_BY_FAIL'} 
admin_dict = {0 : 'NONE', 1 : 'UP', 2 : 'DOWN', 3 : 'UP_ONCE'} 
module_dict = {0 : 'INITIALIZING', 1 : 'PLUGGED', 2 : 'UNPLUGGED', 3: 'PLUGGED-ERR', 4: 'PLUGGED-DIS'}        
oper_speed_dict = {0: 'N/A', 1: '1GB_CX_SGMII', 2: '1GB_KX', 3: '10GB_CX4_XAUI', 4: '10GB_KX4', 5: '10GB_KR', 6: '20GB_KR2', 7: '40GB_CR4',
                   8: '40GB_KR4', 9: '56GB_KR4', 10: '56GB_KX4', 11: '10GB_CR', 12: '10GB_SR', 13: '10GB_ER_LR', 14: '40GB_SR4', 15: '40GB_LR4_ER4',
                   16: '100GB_CR4', 17: '100GB_SR4', 18: '100GB_KR4', 19: '100GB_LR4_ER4', 20: '25GB_CR', 21: '25GB_KR', 22: '25GB_SR', 23: '50GB_CR2',
                   24: '50GB_KR2', 25: '50GB_SR2', 26: 'Not Supported'}
fec_mode_dict = {0: 'Auto', 1: 'None', 2: 'FC', 4: 'RS'}
       
import sys , errno
import os
from test_infra_common import *
from python_sdk_api.sx_api import *
from python_sdk_api.sxd_api import *
import sxd_api_chip_type_rev_get as sxd_api_chip_type
unsupported_phy_mode_devices = [SXD_MGIR_HW_DEV_ID_SX, SXD_MGIR_HW_DEV_ID_SWITCH_IB, SXD_MGIR_HW_DEV_ID_SWITCH_IB2]

ERR_FILE_LOCATION = '/tmp/python_err_log.txt'

file_exist = os.path.isfile(ERR_FILE_LOCATION)
sys.stderr = open(ERR_FILE_LOCATION, 'w')
if not file_exist:
    os.chmod(ERR_FILE_LOCATION, 0o777);

old_stdout = redirect_stdout()
rc, handle = sx_api_open(None)
sys.stdout = os.fdopen(old_stdout, 'w')
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

port_attributes_list = new_sx_port_attributes_t_arr(64)
port_cnt_p = new_uint32_t_p()
max_mtu_size_p=new_sx_port_mtu_t_p()
oper_mtu_size_p=new_sx_port_mtu_t_p()
uint32_t_p_assign(port_cnt_p,64)
oper_state_p = new_sx_port_oper_state_t_p()
admin_state_p = new_sx_port_admin_state_t_p()
module_state_p = new_sx_port_module_state_t_p()
vid_p = new_sx_vid_t_p()
admin_speed_p = new_sx_port_speed_capability_t_p()
oper_speed_p = new_sx_port_oper_speed_t_p()
admin_mode_p = new_sx_port_phy_mode_t_p()
oper_mode_p = new_sx_port_phy_mode_t_p()

rc = sx_api_port_device_get(handle, 1 , 0, port_attributes_list,  port_cnt_p)
port_cnt = uint32_t_p_value(port_cnt_p)
if (rc != 0):
    print "An error was found in sx_api_fdb_uc_mac_addr_get.\nThe example will exit"
    exit()
print ("=================================================================================================================")
header = ["log_port", "local_port", "label_port", "mtu", "admin_s", "oper_s", "module_s", "pvid", "oper_speed", "fec_mode"]
print "|%10s|%10s|%10s|%10s|%8s|%8s|%15s|%6s|%15s|%10s|" % (header[0],header[1],header[2],header[3],header[4],header[5],header[6], header[7], header[8], header[9])
print ("=================================================================================================================")
device_id, device_hw_revision = sxd_api_chip_type.get_chip_type_and_rev()
for i in range(0,port_cnt):

    port_attributes = sx_port_attributes_t_arr_getitem(port_attributes_list,i)
    is_vport = check_vport(int(port_attributes.log_port))
    is_nve = check_nve(int(port_attributes.log_port))
    if is_nve:
        continue
    else:
        if is_vport:
            sx_port_mtu_t_p_assign(max_mtu_size_p, 0)
            sx_port_mtu_t_p_assign(oper_mtu_size_p, 0)
            pvid = 'N/A'
        else:
            sx_api_port_mtu_get(handle, int(port_attributes.log_port), max_mtu_size_p, oper_mtu_size_p)
            sx_api_vlan_port_pvid_get(handle, int(port_attributes.log_port), vid_p)
            pvid = sx_vid_t_p_value(vid_p)
        sx_api_port_state_get(handle, int(port_attributes.log_port), oper_state_p, admin_state_p, module_state_p)
        sx_api_port_speed_get(handle, int(port_attributes.log_port), admin_speed_p, oper_speed_p)
        # The speed is ignored when getting the operational mode
        if device_id in unsupported_phy_mode_devices:
            oper_mode = sx_port_phy_mode_t()
            oper_mode.fec_mode = SX_PORT_FEC_MODE_NONE
        else:
            sx_api_port_phy_mode_get(handle, int(port_attributes.log_port), SX_PORT_PHY_SPEED_10GB, admin_mode_p, oper_mode_p)
            oper_mode = sx_port_phy_mode_t_p_value(oper_mode_p)
    log_port = "0x%x" % port_attributes.log_port
    print ("|%10s|%10d|%10d|%10d|%8s|%8s|%15s|%6s|%15s|%10s|" % 
                                                                (log_port, 
                                                                 port_attributes.port_mapping.local_port,
                                                                 port_attributes.port_mapping.module_port + 1,
                                                                 sx_port_mtu_t_p_value(oper_mtu_size_p),
                                                                 admin_dict[sx_port_admin_state_t_p_value(admin_state_p)],
                                                                 oper_dict[sx_port_oper_state_t_p_value(oper_state_p)],
                                                                 module_dict[sx_port_module_state_t_p_value(module_state_p)],
                                                                 pvid,
                                                                 oper_speed_dict[sx_port_oper_speed_t_p_value(oper_speed_p)],
                                                                 fec_mode_dict[oper_mode.fec_mode]))

print ("=================================================================================================================")


