#!/usr/bin/env python
"""
This file contains Python command example for the RESOURCE MANAGER module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below performs the following configurations:
	1. configure the number of data entry duplication per table type
	2. 
	
"""
import sys, socket, struct, errno

from python_sdk_api.sx_api import *
import sys
from python_sdk_api.sxd_api import *

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

######################################################
#    defines 
######################################################


######################################################
#    functions 
######################################################

def get_chip_type():
    sxd_access_reg_init(0, None, SX_VERBOSITY_LEVEL_INFO)

    mgir = ku_mgir_reg()

    meta = sxd_reg_meta_t()
    meta.dev_id = 1
    meta.swid = 0
    meta.access_cmd = SXD_ACCESS_CMD_GET

    rc = sxd_access_reg_mgir(mgir, meta, 1, None, None)
    assert 0 == rc

    #print ("mgir_reg.hw_info.device_id = %s" % (hex(mgir.hw_info.device_id)))
    #print ("mgir_reg.hw_info.device_hw_revision = %s" % (hex(mgir.hw_info.device_hw_revision)))
    #print ("SXD_MGIR_HW_DEV_ID_SPECTRUM2 = %s" % (hex(SXD_MGIR_HW_DEV_ID_SPECTRUM2)))

    return mgir.hw_info.device_id


def rm_single_table_duplication_parameters_get(resource):
    res = {}

    param_p = new_sx_sdk_table_duplication_param_t_p()
    param_p.sub_type = SX_SDK_SUB_TYPE_NONE
    param_p.num_of_duplication = 0

    rc = sx_api_rm_entries_duplication_get(handle, resource, param_p)
    assert rc == SX_STATUS_SUCCESS, "Failed to get parameters for %d resource type." % (resource)

    res['resource'] = resource
    res['sub_type'] = param_p.sub_type
    res['num_of_duplication'] = param_p.num_of_duplication

    delete_sx_sdk_table_duplication_param_t_p(param_p)
    return res

def rm_single_table_duplication_parameters_set(resource, num_of_duplication):
    param_p = new_sx_sdk_table_duplication_param_t_p()

    param_p.num_of_duplication = num_of_duplication

    if (resource == RM_SDK_TABLE_TYPE_FIB_IPV4_UC_E) | \
       (resource == RM_SDK_TABLE_TYPE_FIB_IPV6_UC_E):
        param_p.sub_type = SX_SDK_SUB_TYPE_DEFAULT
    else:
        param_p.sub_type = SX_SDK_SUB_TYPE_NONE

    rc = sx_api_rm_entries_duplication_set(handle, resource, param_p)

    delete_sx_sdk_table_duplication_param_t_p(param_p)
    return rc



######################################################
#    main
######################################################
def main():

    if get_chip_type() != 0xcf6c:  # SXD_MGIR_HW_DEV_ID_SPECTRUM2
        print "\n\tDynamic Data Duplciation is supported only on Spectrum2 chip type, hence go out from example.\n"
    else:
        # Get the current number of entry duplication for specified resource type
        resource = RM_SDK_TABLE_TYPE_NVE_MC_LIST_E
        original_param = rm_single_table_duplication_parameters_get(resource)
        assert bool(original_param), "Failed to get %d resource parameters." % (resource)

        # Change the number of duplication for specified resource type
        num_of_duplication_to_set = SX_SDK_ENTRY_DUPLICATION_4
        rc = rm_single_table_duplication_parameters_set(resource, num_of_duplication_to_set)
        assert rc == SX_STATUS_SUCCESS, "Failed to configure the number of duplication for %d resource, rc = %d" % (resource, rc)

        # Restore original configuration
        rc = rm_single_table_duplication_parameters_set(resource, original_param["num_of_duplication"])
        assert SX_STATUS_SUCCESS == rc, "Failed to restore original duplication configuration, rc = %d" % (rc)

if __name__ == "__main__":
    main()

