#!/usr/bin/env python
'''
This file contains Python command example for the ROUTER NEIGHBORS DUMP module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.
'''
import sys, errno
import os

from python_sdk_api.sx_api import *
from pprint import pprint
from test_infra_common import *

ip_dict = {0 : 'NONE', 1 : 'IPV4', 2 : 'IPV6'}
prio_dict = {0 : 'BEST_EFFORT', 1 : 'LOW', 2 : 'MED', 3 : 'HIGH', 4 : 'CRITICAL'}
action_dict = {0 : 'DROP', 1 : 'TRAP', 2 : 'FORWARD', 3 : 'TRAP_FORWARD', 4 : 'SPAN'}
ERR_FILE_LOCATION = '/tmp/python_err_log.txt'

file_exist = os.path.isfile(ERR_FILE_LOCATION)
sys.stderr = open(ERR_FILE_LOCATION, 'w')
if not file_exist:
    os.chmod(ERR_FILE_LOCATION, 0o777);
    
old_stdout = redirect_stdout()
rc, handle = sx_api_open(None)
sys.stdout = os.fdopen(old_stdout, 'w')
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

def get_all_neighbors(version):
    neighbors = []
    neigh_entry_arr = new_sx_neigh_get_entry_t_arr(64)
    rif =  0
    neigh_key_p = new_sx_ip_addr_t_p()
    neigh_key = sx_ip_addr_t()
    neigh_key.version = version
    neigh_key.addr.ipv4.addr = 0
    sx_ip_addr_t_p_assign(neigh_key_p, neigh_key)
    data_cnt_p = new_uint32_t_p()
    uint32_t_p_assign(data_cnt_p,20)
    rc = sx_api_router_neigh_get(handle, SX_ACCESS_CMD_GET_FIRST, rif, neigh_key_p, None, neigh_entry_arr, data_cnt_p)
    data_cnt = uint32_t_p_value(data_cnt_p)
    if rc != 0:
        #check if router module initialize
        if rc == SX_STATUS_MODULE_UNINITIALIZED:
            print "####################################"
            print "# Router is not initialized " 
            print "####################################"
            exit()
        print "An error was found in sx_api_router_neigh_get. rc: %d" % (rc)
        exit()

    read_number = 0
    while (data_cnt == 20):
        for i in range(0, data_cnt):
            neighbor = sx_neigh_get_entry_t_arr_getitem(neigh_entry_arr, i)
            #print_all_attr_from_neighbor(neighbor)
            neighbors.append(neighbor)
     
        rif = neighbor.neigh_data.rif
        sx_ip_addr_t_p_assign(neigh_key_p, neighbor.ip_addr)        
        rc = sx_api_router_neigh_get(handle, SX_ACCESS_CMD_GETNEXT, rif, neigh_key_p, None, neigh_entry_arr, data_cnt_p)
        print ("sx_api_router_neigh_get: rc %d " % ( rc) )
        data_cnt = uint32_t_p_value(data_cnt_p)
        read_number = read_number + 1
        
    for i in range(0, data_cnt):
        neighbor = sx_neigh_get_entry_t_arr_getitem(neigh_entry_arr, i)
        neighbors.append(neighbor)

    return neighbors
        
neighbors = get_all_neighbors(SX_IP_VERSION_IPV4) + get_all_neighbors(SX_IP_VERSION_IPV6)
if len(neighbors) > 0:
    print "----------------------------------------------------------------------------------------------------------"
    print "|%10s|%40s|%15s|%15s|%20s|" % ("RIF", "IP", "Action", "Trap Prio", "MAC")
    print "----------------------------------------------------------------------------------------------------------"  
    for neighbor in neighbors:
        print "|%10d|%40s|%15s|%15s|%20s|" % (neighbor.neigh_data.rif,
                                              ip_addr_to_str(neighbor.ip_addr),
                                              action_dict[neighbor.neigh_data.action],
                                              prio_dict[neighbor.neigh_data.trap_attr.prio],
                                              neighbor.neigh_data.mac_addr.to_str())
        print "----------------------------------------------------------------------------------------------------------"









