#!/usr/bin/env python
'''
This file contains Python command example for the Span module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of different Span attributes.
This example is supported on Spectrum devices. 
'''
import sys
import errno, sys
import colorsys
from python_sdk_api.sx_api import *
from test_infra_common import *

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

""" ############################################################################################ """
port_list = mapPortAndInterfaces(handle)
PORT_1 = port_list[0]
PORT_2 = port_list[1]
PORT_3 = port_list[2]

""" ############################################################################################ """
def span_init() :
    """SPAN INIT"""
    print ("--------------- SPAN INIT------------------------------")
    
    init_params_p = new_sx_span_init_params_t_p()
    #init_params_p->version = SX_SPAN_MIRROR_HEADER_NONE
    init_params = sx_span_init_params_t()
    init_params.version = SX_SPAN_MIRROR_HEADER_NONE
    sx_span_init_params_t_p_assign(init_params_p, init_params)
    rc = sx_api_span_init_set(handle, init_params_p)
    print("sx_api_span_init_set [rc=%d]" % (rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    print ("version = %d " % ( init_params.version) )
    delete_sx_span_init_params_t_p(init_params_p)

""" ############################################################################################ """
def span_session_create(span_session_param_p) :
    """SPAN SESSION CREATE"""
    print ("--------------- SPAN SESSION CREATE------------------------------")
    
    span_session_id_p = new_sx_span_session_id_t_p()

    rc = sx_api_span_session_set(handle, SX_ACCESS_CMD_CREATE, span_session_param_p,
                                 span_session_id_p)
    print("sx_api_span_session_set [rc=%d]" %(rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
      
    return span_session_id_p
    
""" ############################################################################################ """

def span_session_edit(span_type, span_session_id_p):
    """SPAN SESSION EDIT"""
    print ("--------------- SPAN SESSION EDIT------------------------------")
    span_session_param_p = new_sx_span_session_params_t_p()
    span_session_param = sx_span_session_params_t()
    if span_type == SX_SPAN_TYPE_REMOTE_ETH_VLAN_TYPE1:
        span_session_param.span_type = span_type
        span_session_param.span_type_format.remote_eth_vlan_type1.qos_mode = SX_SPAN_QOS_MAINTAIN
        span_session_param.span_type_format.remote_eth_vlan_type1.switch_prio = 5
        span_session_param.span_type_format.remote_eth_vlan_type1.vid = 3876
        span_session_param.span_type_format.remote_eth_vlan_type1.vlan_ethertype_id = 0
        span_session_param.span_type_format.remote_eth_vlan_type1.pcp = 4
        span_session_param.span_type_format.remote_eth_vlan_type1.dei = 1
        span_session_param.truncate = True
        span_session_param.truncate_size =  100
    sx_span_session_params_t_p_assign(span_session_param_p, span_session_param)
    
    
    rc = sx_api_span_session_set(handle, SX_ACCESS_CMD_EDIT, span_session_param_p,
                                 span_session_id_p)
    print("sx_api_span_session_set [rc=%d]" %(rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    
    span_session_id = sx_span_session_id_t_p_value(span_session_id_p)
    print("Span session id [%d] Edited" %(span_session_id))
    delete_sx_span_session_params_t_p(span_session_param_p)
""" ############################################################################################ """

def span_session_destroy(span_session_id_p, span_session_param_p):
    """SPAN SESSION DESTROY"""
    print ("--------------- SPAN SESSION DESTROY------------------------------")
        
    rc = sx_api_span_session_set(handle, SX_ACCESS_CMD_DESTROY, span_session_param_p,
                                 span_session_id_p)
    print("sx_api_span_session_set [rc=%d]" %(rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    
    span_session_id = sx_span_session_id_t_p_value(span_session_id_p)
    print("Span session id [%d] destroyed" %(span_session_id))
    
    
""" ############################################################################################ """

def span_analyzer_set(cmd, log_port, port_paramas_p, session_id):
    """SPAN ANALYZER ADD/DELETE"""
    if cmd == SX_ACCESS_CMD_ADD:
        print ("--------------- SPAN ANALYZER ADD ------------------------------")
    else:
        print ("--------------- SPAN ANALYZER DELETE ------------------------------")
    rc = sx_api_span_analyzer_set(handle, cmd, log_port, port_paramas_p, session_id)
    print("sx_api_span_analyzer_set [rc=%d]" %(rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)

""" ############################################################################################ """

def span_analyzer_get(log_port):
    """SPAN ANALYZER GET"""
    print ("--------------- SPAN ANALYZER GET ------------------------------")
    span_sessions_cnt_p = new_uint32_t_p()
    """ Set cnt = 1 """
    uint32_t_p_assign(span_sessions_cnt_p, 1) 
    """ Get value from PTR """    
    session_count = uint32_t_p_value(span_sessions_cnt_p)
    
    port_params_p = new_sx_span_analyzer_port_params_t_arr(session_count)
    session_id_list_p = new_sx_span_session_id_t_arr(session_count)
    
    rc = sx_api_span_analyzer_get(handle, log_port, port_params_p, session_id_list_p, span_sessions_cnt_p )
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    print("sx_api_span_analyzer_get [rc=%d]" %(rc))
    #port_params = sx_span_analyzer_port_params_t()
    
    for i in range(0,session_count):
        session_id = sx_span_session_id_t_arr_getitem(session_id_list_p, i)
        port_params = sx_span_analyzer_port_params_t_arr_getitem(port_params_p, i) 
        print("cng mng =  %d" %(port_params.cng_mng))
    
    delete_uint32_t_p(span_sessions_cnt_p)
    delete_sx_span_analyzer_port_params_t_arr(port_params_p)
    delete_sx_span_session_id_t_arr(session_id_list_p)

""" ############################################################################################ """

def span_mirror_set(cmd, mirror_port, mirror_direction, session_id):
    """ADD/DELETE SPAN MIRROR PORT """
    if cmd == SX_ACCESS_CMD_ADD:
        print ("--------------- ADD SPAN MIRROR PORT ------------------------------")
    else:
        print ("--------------- DELETE SPAN MIRROR PORT  ------------------------------")
    rc = sx_api_span_mirror_set(handle, cmd, mirror_port, mirror_direction, session_id)
    print("sx_api_span_mirror_set [rc=%d]" %(rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
""" ############################################################################################ """

def span_mirror_get(mirror_port, mirror_direction):
    """SPAN MIRROR PORT GET """
    print ("--------------- SPAN MIRROR PORT GET ------------------------------")

    session_id_p = new_sx_span_session_id_t_p()
    rc = sx_api_span_mirror_get(handle, mirror_port, mirror_direction, session_id_p)
    
    session_id = sx_span_session_id_t_p_value(span_session_id_p)
    print("sx_api_span_mirror_get [rc=%d], span session id = %d" %(rc, session_id))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES) 
    
    delete_sx_span_session_id_t_p(session_id_p)    
                   
""" ############################################################################################ """

def span_session_state_set(session_id, admin_state):
    """SPAN SESSION STATE SET """
    print ("--------------- SPAN SESSION STATE SET  ------------------------------")
    print("Span session id =%d, admin state =%d" %(session_id, admin_state))
    rc = sx_api_span_session_state_set(handle, session_id, admin_state)
    print("sx_api_span_session_state_set [rc=%d]" %(rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
                
""" ############################################################################################ """

def span_mirror_state_set(mirror_port, mirror_direction, admin_state):
    """SPAN MIRROR STATE SET """
    print ("--------------- SPAN MIRROR STATE SET  ------------------------------")
    print("Mirror port = 0x%x " %(mirror_port))
    print("Mirror direction =%d, admin state =%d" %(mirror_direction, admin_state))
    rc = sx_api_span_mirror_state_set(handle, mirror_port, mirror_direction, admin_state)
    print("sx_api_span_mirror_state_set [rc=%d]" %(rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    
""" ############################################################################################ """

def span_mirror_state_get(mirror_port, mirror_direction):
    """SPAN MIRROR STATE GET """
    print ("--------------- SPAN MIRROR STATE GET  ------------------------------")
    
    admin_state_p = new_boolean_t_p()
    rc = sx_api_span_mirror_state_get(handle, mirror_port, mirror_direction, admin_state_p)
    print("sx_api_span_mirror_state_get [rc=%d]" %(rc))
    
    admin_state = boolean_t_p_value(admin_state_p)
    print("Mirror port = 0x%x " %(mirror_port))
    print("Mirror direction =%d, admin state =%d" %(mirror_direction, admin_state))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    delete_boolean_t_p(admin_state_p)
        
""" ############################################################################################ """
   
def span_mirror_tables_set(cmd, session_id):
    """ SPAN MIRROR TABLE ADD/DELETE"""
    if cmd == SX_ACCESS_CMD_ADD:
        print ("------------ SPAN MIRROR TABLE ADD -----------------")
    else:
        print ("------------ SPAN MIRROR TABLE DELETE -----------------")
    
    rc = sx_api_span_mirror_tables_set(handle, cmd, session_id) 
    print("sx_api_span_mirror_tables_set [rc=%d]" %(rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)   

""" ############################################################################################ """
   
def span_mirror_tables_get():
    """ SPAN MIRROR TABLE GET"""
    print ("------------ SPAN MIRROR TABLE GET -----------------")
    
    session_id_p =  new_sx_span_session_id_t_p()
    rc = sx_api_span_mirror_tables_get(handle, session_id_p)
    session_id = sx_span_session_id_t_p_value(session_id_p) 
    print("sx_api_span_mirror_tables_get [rc=%d], span session_id = %d" %(rc, session_id))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)  
    delete_sx_span_session_id_t_p(session_id_p)
""" ############################################################################################ """

def span_drop_mirror_set(cmd, session_id, drop_reason_list=[], drop_mirroring_attr_p=None):
    """ SPAN DROP MIRROR SET/ADD/DELETE/DELETE-ALL"""
    cmd_str = { SX_ACCESS_CMD_SET:        "SET",
                SX_ACCESS_CMD_ADD:        "ADD",
                SX_ACCESS_CMD_DELETE:     "DELETE",
                SX_ACCESS_CMD_DELETE_ALL: "DELETE-ALL" }
    if cmd in cmd_str:
        print ("------------ SPAN DROP MIRROR %s -----------------" % (cmd_str[cmd]))
    else:
        print ("------------ SPAN DROP MIRROR cmd %u unsupported -----------------" %(cmd))

    drop_reason_count = len(drop_reason_list)
    drop_reason_list_arr = new_sx_span_drop_reason_t_arr(drop_reason_count)
    for idx in range(drop_reason_count):
        sx_span_drop_reason_t_arr_setitem(drop_reason_list_arr, idx, drop_reason_list[idx])

    rc = sx_api_span_drop_mirror_set(handle, cmd, session_id, drop_mirroring_attr_p,
                                     drop_reason_list_arr, drop_reason_count)
    print("sx_api_span_drop_mirror_set [rc=%d]" %(rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    delete_sx_span_drop_reason_t_arr(drop_reason_list_arr)

""" ############################################################################################ """

def span_drop_mirror_get(session_id):
    """ SPAN MIRROR TABLE GET"""
    print ("------------ SPAN DROP MIRROR GET -----------------")

    drop_reason_count_p = new_uint32_t_p()
    uint32_t_p_assign(drop_reason_count_p, 0)
    rc = sx_api_span_drop_mirror_get(handle, session_id, None, None, drop_reason_count_p)
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    drop_reason_count = uint32_t_p_value(drop_reason_count_p)
    print("sx_api_span_drop_mirror_get [rc=%d], drop_reason_count = %d" %(rc, drop_reason_count))

    drop_reason_list = []
    drop_mirroring_attr_p = None
    uint32_t_p_assign(drop_reason_count_p, drop_reason_count)
    drop_reason_list_arr = new_sx_span_drop_reason_t_arr(drop_reason_count)
    rc = sx_api_span_drop_mirror_get(handle, session_id, drop_mirroring_attr_p,
                                     drop_reason_list_arr, drop_reason_count_p)
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    if uint32_t_p_value(drop_reason_count_p) != drop_reason_count:
        print("drop reason count mismatch: %d != %d"
              % (uint32_t_p_value(drop_reason_count_p), drop_reason_count))
        sys.exit(errno.EACCES)
    for idx in range(drop_reason_count):
        print("    drop reason: %u" % (sx_span_drop_reason_t_arr_getitem(drop_reason_list_arr, idx)))
    drop_reason_list = [ sx_span_drop_reason_t_arr_getitem(drop_reason_list_arr, idx) for idx in range(drop_reason_count) ]
    delete_uint32_t_p(drop_reason_count_p)
    delete_sx_span_drop_reason_t_arr(drop_reason_list_arr)
    return drop_reason_list

""" ############################################################################################ """

def span_deinit():
    """SPAN DEINIT"""
    print ("--------------- SPAN DEINIT------------------------------")
    
    rc = sx_api_span_deinit_set(handle)
    print("sx_api_span_deinit_set [rc=%d]" % (rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
        
""" ############################################################################################ """

def span_session_iter_get(cmd, session_key, filter_p, ret_session_list_p, session_cnt_p):
    """SPAN Session Iter Get"""
    print ("--------------- SPAN SESSION ITER GET------------------------------")
    
    session_key_p = new_sx_span_session_id_t_p()
    sx_span_session_id_t_p_assign(session_key_p, session_key)
    rc = sx_api_span_session_iter_get(handle, cmd, session_key_p, filter_p, ret_session_list_p, session_cnt_p)
    print("sx_api_span_session_iter_get [rc=%d]" % (rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
        
""" ############################################################################################ """

#init span
span_init()
#create a span session
span_session_param_p = new_sx_span_session_params_t_p()
span_session_param = sx_span_session_params_t()
span_session_param.span_type = SX_SPAN_TYPE_LOCAL_ETH_TYPE1
span_session_param.span_type_format.local_eth_type1.qos_mode = SX_SPAN_QOS_MAINTAIN
span_session_param.span_type_format.local_eth_type1.switch_prio = 2
span_session_param.truncate = False
span_session_param.truncate_size =  32
sx_span_session_params_t_p_assign(span_session_param_p, span_session_param)
span_session_id_p = span_session_create(span_session_param_p)
span_session_id = sx_span_session_id_t_p_value(span_session_id_p)
print("Span session id [%d] created" %(span_session_id))

#add analyzer port to span session created above
port_params_p = new_sx_span_analyzer_port_params_t_p()
port_params = sx_span_analyzer_port_params_t()                                                 
port_params.cng_mng  = SX_SPAN_CNG_MNG_DISCARD
sx_span_analyzer_port_params_t_p_assign(port_params_p, port_params)
span_analyzer_set(SX_ACCESS_CMD_ADD, PORT_1,port_params_p, span_session_id)
span_analyzer_get(PORT_1)

#add span mirror port
span_mirror_set(SX_ACCESS_CMD_ADD, PORT_2, SX_SPAN_MIRROR_INGRESS, span_session_id)
span_mirror_set(SX_ACCESS_CMD_ADD, PORT_3, SX_SPAN_MIRROR_INGRESS, span_session_id)

#span mirror get
span_mirror_get(PORT_3, SX_SPAN_MIRROR_INGRESS)

#enable SPAN session admin state
span_session_state_set(span_session_id, True)

#enable SPAN mirror state
span_mirror_state_set(PORT_2, SX_SPAN_MIRROR_INGRESS, True)
span_mirror_state_set(PORT_3, SX_SPAN_MIRROR_INGRESS, True)

#get SPAN mirror state
span_mirror_state_get(PORT_3, SX_SPAN_MIRROR_INGRESS)

#enable table mirroring - for registers without explicit session
span_mirror_tables_set(SX_ACCESS_CMD_ADD, span_session_id)

#mirror table get
span_mirror_tables_get()

#disable table mirroring
span_mirror_tables_set(SX_ACCESS_CMD_DELETE, span_session_id)

#disable SPAN mirror state
span_mirror_state_set(PORT_2, SX_SPAN_MIRROR_INGRESS, False)
span_mirror_state_set(PORT_3, SX_SPAN_MIRROR_INGRESS, False)

#delete span mirror port
span_mirror_set(SX_ACCESS_CMD_DELETE, PORT_2, SX_SPAN_MIRROR_INGRESS, span_session_id)
span_mirror_set(SX_ACCESS_CMD_DELETE, PORT_3, SX_SPAN_MIRROR_INGRESS, span_session_id)

#disable SPAN session admin state
span_session_state_set(span_session_id, False)

#delete analyzer port to span session created above
span_analyzer_set(SX_ACCESS_CMD_DELETE, PORT_1, port_params_p, span_session_id)

#iteratively GET the active SPAN session IDs
span_session_cnt = 2
span_session_cnt_p = new_uint32_t_p()
uint32_t_p_assign(span_session_cnt_p, span_session_cnt)
session_id_list_p = new_sx_span_session_id_t_arr(span_session_cnt)
span_session_iter_get(SX_ACCESS_CMD_GET_FIRST, span_session_id, None, session_id_list_p,span_session_cnt_p )
#####################################################################################

#edit span session created above
span_type = SX_SPAN_TYPE_REMOTE_ETH_VLAN_TYPE1
span_session_edit(span_type, span_session_id_p)

#add analyzer port to span session created above
span_analyzer_set(SX_ACCESS_CMD_ADD, PORT_1,port_params_p, span_session_id)

#add span mirror port
span_mirror_set(SX_ACCESS_CMD_ADD, PORT_2, SX_SPAN_MIRROR_INGRESS, span_session_id)
span_mirror_set(SX_ACCESS_CMD_ADD, PORT_3, SX_SPAN_MIRROR_INGRESS, span_session_id)

#span mirror get
span_mirror_get(PORT_2, SX_SPAN_MIRROR_INGRESS)

#enable SPAN session admin state
span_session_state_set(span_session_id, True)

#enable SPAN mirror state
span_mirror_state_set(PORT_2, SX_SPAN_MIRROR_INGRESS, True)
span_mirror_state_set(PORT_3, SX_SPAN_MIRROR_INGRESS, True)

#get SPAN mirror state
span_mirror_state_get(PORT_2, SX_SPAN_MIRROR_INGRESS)

#enable table mirroring - for registers without explicit session
span_mirror_tables_set(SX_ACCESS_CMD_ADD, span_session_id)

#mirror table get
span_mirror_tables_get()

#disable table mirroring
span_mirror_tables_set(SX_ACCESS_CMD_DELETE, span_session_id)

#set drop mirroring drop reasons
span_drop_mirror_set(SX_ACCESS_CMD_ADD, span_session_id,
                     drop_reason_list=[SX_SPAN_DROP_REASON_ALL_ROUTER_DROPS_E])
#get drop mirroring
span_drop_mirror_get(span_session_id)

#delete all drop mirroring drop reasons
span_drop_mirror_set(SX_ACCESS_CMD_DELETE_ALL, span_session_id)

#disable SPAN mirror state
span_mirror_state_set(PORT_2, SX_SPAN_MIRROR_INGRESS, False)
span_mirror_state_set(PORT_3, SX_SPAN_MIRROR_INGRESS, False)

#delete span mirror port
span_mirror_set(SX_ACCESS_CMD_DELETE, PORT_2, SX_SPAN_MIRROR_INGRESS, span_session_id)
span_mirror_set(SX_ACCESS_CMD_DELETE, PORT_3, SX_SPAN_MIRROR_INGRESS, span_session_id)

#disable SPAN session admin state
span_session_state_set(span_session_id, False)

#delete analyzer port to span session created above
span_analyzer_set(SX_ACCESS_CMD_DELETE, PORT_1,port_params_p, span_session_id)

#destroy span session
span_session_destroy(span_session_id_p, span_session_param_p)
delete_sx_span_session_params_t_p(span_session_param_p)
delete_sx_span_session_id_t_p(span_session_id_p)
delete_sx_span_analyzer_port_params_t_p(port_params_p)

#deinit span 
span_deinit()

""" ############################################################################################ """
sx_api_close(handle)
""" ############################################################################################ """
