#!/usr/bin/env python
"""
This file contains a TestX command example for the VLAN module.
TestX commands syntax is very similar to the SwitchX SDK APIs.
The TestX commands and the corresponding APIs differ only by the fact that the TestX commands hide the parameters which can be managed in the background (such as api_handle, pointers etc).
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below performs the following configuration operations:
1. Adds multiple ports to a specific VLAN.
2. Adds a port to a list of VLANs.
3. Retrieves ports in a specific VLAN.
4. Update port's default VLAN.
5. Sets VLAN related port attributes.
6. Sets port's Q-in-Q mode.
7. Sets port's outer tag priority (relevant only when Q-in-Q is enabled).
8. Removes all the ports from the above mentioned VLANs.

"""

import sys, errno
from python_sdk_api.sx_api import * 
from test_infra_common import *

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

port_list = mapPortAndInterfaces(handle)
PORT_1 = port_list[0]
PORT_2 = port_list[1]

# Adds multiple ports to a specific VLAN.
vlan_arr = new_sx_vlan_ports_t_arr(2)
vlan_port0 = sx_vlan_ports_t()
vlan_port0.log_port = PORT_1
vlan_port0.is_untagged = 1
vlan_port1 = sx_vlan_ports_t()
vlan_port1.log_port = PORT_2
vlan_port1.is_untagged = 1
sx_vlan_ports_t_arr_setitem(vlan_arr, 0, vlan_port0)
sx_vlan_ports_t_arr_setitem(vlan_arr, 1, vlan_port1)
swid = 0
vid = 5
rc = sx_api_vlan_ports_set(handle, SX_ACCESS_CMD_ADD, swid, vid, vlan_arr, 2)
print("Add %d ports to vlan %d , rc: %d " %(2, vid, rc))

# Update port's default VLAN.
log_port = PORT_2                             
pvid = 4                                             
rc = sx_api_vlan_port_pvid_set(handle, SX_ACCESS_CMD_ADD, log_port, pvid);
print ("Set pvid %d for port %d, rc %d " % (pvid, log_port, rc) )        

# Update port's default VLAN.
log_port = PORT_2
pvid = 4
rc = sx_api_vlan_port_pvid_set(handle, SX_ACCESS_CMD_DELETE, log_port, pvid);
print ("Delete pvid %d for port %d, rc %d " % (pvid, log_port, rc) )

#sx_api_vlan_port_prio_tagged_set 0x10100 SX_PRIO_TAGGED_STATE
log_port = PORT_1
prio_tag_state = SX_PRIO_TAGGED_STATE
rc = sx_api_vlan_port_prio_tagged_set(handle, log_port, prio_tag_state);
print ("Set SX_PRIO_TAGGED_STATE prio_tag_state %d for port %d, rc %d " % (prio_tag_state, log_port, rc) )

#sx_api_vlan_port_prio_tagged_set 0x10100 SX_UNTAGGED_STATE
log_port = PORT_1
prio_tag_state = SX_UNTAGGED_STATE
rc = sx_api_vlan_port_prio_tagged_set(handle, log_port, prio_tag_state);
print ("Set prio_tag_state %d for port %d, rc %d " % (prio_tag_state, log_port, rc) )

#sx_api_vlan_port_ingr_filter_set 0x10100 SX_INGR_FILTER_DISABLE
log_port = PORT_1
ingr_fltr_state = SX_INGR_FILTER_DISABLE
rc = sx_api_vlan_port_ingr_filter_set(handle, log_port, ingr_fltr_state);
print ("Set ingr_fltr_state %d for port %d, rc %d " % (ingr_fltr_state, log_port, rc) )

#sx_api_vlan_port_ingr_filter_set 0x10100 SX_INGR_FILTER_ENABLE
log_port = PORT_1
ingr_fltr_state = SX_INGR_FILTER_ENABLE
rc = sx_api_vlan_port_ingr_filter_set(handle, log_port, ingr_fltr_state);
print ("Set ingr_fltr_state %d for port %d, rc %d " % (ingr_fltr_state, log_port, rc) )

#sx_api_vlan_port_accptd_frm_types_set PORT_1 {1;0;0}
log_port = PORT_1
accptd_frame_type = sx_vlan_frame_types_t()
accptd_frame_type.allow_untagged = 1
accptd_frame_type.allow_priotagged = 0
accptd_frame_type.allow_tagged = 0
rc = sx_api_vlan_port_accptd_frm_types_set(handle, log_port, accptd_frame_type);
print ("Set sx_api_vlan_port_accptd_frm_types_set (1,0,0) for port %d, rc %d " % (log_port, rc) )

#sx_api_vlan_port_accptd_frm_types_set PORT_1 {0;1;0}
log_port = PORT_1
accptd_frame_type = sx_vlan_frame_types_t()
accptd_frame_type.allow_untagged = 0
accptd_frame_type.allow_priotagged = 1
accptd_frame_type.allow_tagged = 0
rc = sx_api_vlan_port_accptd_frm_types_set(handle, log_port, accptd_frame_type);
print ("Set sx_api_vlan_port_accptd_frm_types_set (0,1,0) for port %d, rc %d " % (log_port, rc) )

#sx_api_vlan_port_accptd_frm_types_set PORT_1 {0;0;1}
log_port = PORT_1
accptd_frame_type = sx_vlan_frame_types_t()
accptd_frame_type.allow_untagged = 0
accptd_frame_type.allow_priotagged = 0
accptd_frame_type.allow_tagged = 1
rc = sx_api_vlan_port_accptd_frm_types_set(handle, log_port, accptd_frame_type);
print ("sx_api_vlan_port_accptd_frm_types_set (0,0,1) for port %d, rc %d " % (log_port, rc) )

#sx_api_vlan_port_qinq_mode_set 0x10100 SX_QINQ_MODE_QINQ
log_port = PORT_1
qinq_mode = SX_QINQ_MODE_QINQ
#Register SPVTR Unsupported yet by FW
#rc = sx_api_vlan_port_qinq_mode_set(handle, log_port, qinq_mode);
#print ("Set sx_api_vlan_port_qinq_mode_set %d for port %d, rc %d " % (qinq_mode, log_port, rc) )

#sx_api_vlan_port_qinq_outer_prio_mode_set 0x10100 SX_QINQ_OUTER_PRIO_MODE_CVLAN
log_port = PORT_1
qinq_outer_prio_mode = SX_QINQ_OUTER_PRIO_MODE_CVLAN

#Register SPVTR Unsupported yet by FW
#rc = sx_api_vlan_port_qinq_outer_prio_mode_set(handle, log_port, qinq_outer_prio_mode);
#print ("sx_api_vlan_port_qinq_outer_prio_mode_set %d for port %d, rc %d " % (qinq_outer_prio_mode, log_port, rc) )

#sx_api_vlan_default_vid_set 0 1
rc = sx_api_vlan_default_vid_set(handle, 0, 1);
print ("sx_api_vlan_default_vid_set %d for swid %d, rc %d " % (1, 0, rc) )

