#!/usr/bin/env python
"""
This file contains a TestX command example for the VLAN module.
TestX commands syntax is very similar to the SwitchX SDK APIs.
The TestX commands and the corresponding APIs differ only by the fact that the TestX commands hide the parameters which can be managed in the background (such as api_handle, pointers etc).
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below performs the following configuration operations:
1. Adds multiple ports to a specific VLAN.
2. Adds a port to a list of VLANs.
3. Retrieves ports in a specific VLAN.
4. Update port's default VLAN.
5. Sets VLAN related port attributes.
6. Sets port's Q-in-Q mode.
7. Sets port's outer tag priority (relevant only when Q-in-Q is enabled).
8. Removes all the ports from the above mentioned VLANs.

"""

import sys, errno
import errno, sys

from python_sdk_api.sx_api import * 
from test_infra_common import *

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

port_list = mapPortAndInterfaces(handle)
log_port = port_list[0]

# Adds multiple VLANs to a specific Port.
max_vlan=4094
vlan_arr = new_sx_port_vlans_t_arr(max_vlan)
port_vlan = sx_port_vlans_t()

vlan_arr_one = new_sx_vlan_ports_t_arr(1)
vlan_port = sx_vlan_ports_t()
vlan_port.log_port = log_port
vlan_port.is_untagged = 1
sx_vlan_ports_t_arr_setitem(vlan_arr_one, 0, vlan_port)


for i in range(2,max_vlan):
    sx_api_vlan_ports_set(handle, SX_ACCESS_CMD_ADD, 0, i, vlan_arr_one, 1)

for i in range(2,max_vlan):
    sx_api_vlan_ports_set(handle, SX_ACCESS_CMD_DELETE, 0, i, vlan_arr_one, 1)

print("ADD & DELETE - : END of 1 by 1 NOT MULTI FUNC ")


""" -----------------------------------------------------"""
for i in range(2,max_vlan):
    port_vlan.vid = i
    port_vlan.is_untagged = 1
    sx_port_vlans_t_arr_setitem(vlan_arr, 0, port_vlan)
    rc = sx_api_vlan_port_multi_vlan_set(handle, SX_ACCESS_CMD_ADD, log_port, vlan_arr, 1)
    print ("sx_api_vlan_port_multi_vlan_set [rc=%d] " % ( rc) )
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)

print("ADD: END of 1 by 1 setting ")

for i in range(2,max_vlan):
    port_vlan.vid = i
    port_vlan.is_untagged = 1
    sx_port_vlans_t_arr_setitem(vlan_arr, 0, port_vlan)
    rc = sx_api_vlan_port_multi_vlan_set(handle, SX_ACCESS_CMD_DELETE, log_port, vlan_arr, 1)
    print ("sx_api_vlan_port_multi_vlan_set [rc=%d] " % ( rc) )
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
print("DELETE: END of 1 by 1 setting ")

""" USE MULTI VLANS """
for i in range(2,max_vlan):
    port_vlan.vid = i
    port_vlan.is_untagged = 1
    sx_port_vlans_t_arr_setitem(vlan_arr, i-2, port_vlan)

rc = sx_api_vlan_port_multi_vlan_set(handle, SX_ACCESS_CMD_ADD, log_port, vlan_arr, max_vlan-2)
print ("sx_api_vlan_port_multi_vlan_set [rc=%d] " % ( rc) )
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)
print("Add VLANs 2-%d to port 0x%x , rc: %d " %(max_vlan-1, log_port, rc))

rc = sx_api_vlan_port_multi_vlan_set(handle, SX_ACCESS_CMD_DELETE, log_port, vlan_arr, max_vlan-2)
print ("sx_api_vlan_port_multi_vlan_set [rc=%d] " % ( rc) )
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)
print("Delete VLANs 2-%d to port 0x%x , rc: %d " %(max_vlan-1, log_port, rc))

sx_api_close(handle)

