#!/usr/bin/env python
# -*- python -*-

#######################################################################
# global params
#######################################################################
PORT_TYPE_NVE = 8
PORT_TYPE_VPORT = 2
PORT_TYPE_OFFSET = 28
PORT_TYPE_MASK = 0xF0000000
PORT_OPER_STATUS = 0
PORT_ADMIN_STATUS = 1
MAX_COUNT_SLEEPS = 100
#######################################################################
# functions
#######################################################################
from python_sdk_api.sx_api import *
import sys, os, socket, struct
import time

def check_vport(port):
    port_type = (port & PORT_TYPE_MASK) >> PORT_TYPE_OFFSET
    return port_type & PORT_TYPE_VPORT
def check_nve(port):
    port_type = (port & PORT_TYPE_MASK) >> PORT_TYPE_OFFSET
    return port_type & PORT_TYPE_NVE
    
    
    
def mapPortAndInterfaces(handle, num_ports = 4):
    
    count_sleeps = 0

    port_with_admin_up_list = getAllPortWithLinkUp(handle, PORT_ADMIN_STATUS)

    if num_ports > len(port_with_admin_up_list):
        raise Exception("Not enough ports in state up")

    while count_sleeps < MAX_COUNT_SLEEPS :
        count_ports_up = 0
        for i in range(len(port_with_admin_up_list)):
            if True == checkPortUp(handle, port_with_admin_up_list[i]) :
                count_ports_up = count_ports_up + 1

        if count_ports_up >= num_ports :
            break

        time.sleep(1)
        count_sleeps = count_sleeps + 1

    if count_sleeps == MAX_COUNT_SLEEPS :
        raise Exception("Not enough ports in state up")

    port_with_link_up_list = getAllPortWithLinkUp(handle)
    print '[{}]'.format(', '.join(hex(x) for x in port_with_link_up_list))


    return port_with_link_up_list

def wait_till_port_list_up(handle, interface_list, timeout=50):
    for index in range(4):
        wait_till_port_up(handle, interface_list[index].get_log_port(), timeout)
    return True
    
    
def getPortDown(handle, port_list):
    for port in port_list :
        if checkPortDown(handle, port):
            return port
    raise Exception("no port in state down")

def wait_till_port_up(handle, port, timeout = 50):
    start = time.time()
    end   = time.time()
    
    while not checkPortUp(handle,port):
        end = time.time()
        if timeout < end - start:
            print (" \n", end - start)
            sys.exit(1)

    print (" \n", end - start)

    return True

def checkPortUp(handle, port):

    oper_state_t_p = new_sx_port_oper_state_t_p()
    admin_state_t_p = new_sx_port_admin_state_t_p()
    module_state_t_p = new_sx_port_module_state_t_p()

    rc = sx_api_port_state_get(handle, port, oper_state_t_p, admin_state_t_p, module_state_t_p)
    if rc != 0:
        sys.exit(1)

    oper_state = sx_port_oper_state_t_p_value(oper_state_t_p)
    return (oper_state == SX_PORT_OPER_STATUS_UP)

def checkPortDown(handle, port):

    oper_state_t_p = new_sx_port_oper_state_t_p()
    admin_state_t_p = new_sx_port_admin_state_t_p()
    module_state_t_p = new_sx_port_module_state_t_p()
    
    rc = sx_api_port_state_get(handle, port, oper_state_t_p, admin_state_t_p, module_state_t_p)
    if rc != 0:
        sys.exit(1)
    
    oper_state = sx_port_oper_state_t_p_value(oper_state_t_p) 
    return (oper_state == SX_PORT_OPER_STATUS_DOWN)

def getAllPortWithLinkUp(handle, port_mode = PORT_OPER_STATUS):
    
    port_with_link_up_list = []
    
    # Get the active SWID list
    
    swid_list_p = new_sx_swid_t_arr(16)
    
    swid_cnt_p = new_uint32_t_p()
    
    uint32_t_p_assign(swid_cnt_p, 16)
    
    rc = sx_api_port_swid_list_get(handle, swid_list_p, swid_cnt_p)
    if rc != 0:
        sys.exit(1)
    
    swid_cnt = uint32_t_p_value(swid_cnt_p)
    
    #print ("sx_api_port_swid_list_get swid_cnt:%d , rc %d, swid table: " % ( swid_cnt, rc) )
            
    for i in range(0,swid_cnt):
        
        swid = sx_swid_t_arr_getitem(swid_list_p, i)
        
        #print ("[%d] swid %d" % ( i, swid) )
                
        log_port_list_p = new_sx_port_log_id_t_arr(64)
        
        port_cnt_p = new_uint32_t_p()
        
        uint32_t_p_assign(port_cnt_p,64)
        
        rc = sx_api_port_swid_port_list_get(handle, swid, log_port_list_p, port_cnt_p)
        if rc != 0:
            sys.exit(1)
        
        port_cnt = uint32_t_p_value(port_cnt_p)
        
        #print ("sx_api_port_swid_port_list_get port_cnt:%d , rc %d, port table: " % ( port_cnt, rc) )
        
        for i in range(0,port_cnt):
            
            port = sx_port_log_id_t_arr_getitem(log_port_list_p, i)
            
            #print ("[%d] port 0x%x" % ( i, port) )

            oper_state_t_p = new_sx_port_oper_state_t_p()
            admin_state_t_p = new_sx_port_admin_state_t_p()
            module_state_t_p = new_sx_port_module_state_t_p()
            
            rc = sx_api_port_state_get(handle, port, oper_state_t_p, admin_state_t_p, module_state_t_p)
            if rc != 0:
                sys.exit(1)
            
            oper_state = sx_port_oper_state_t_p_value(oper_state_t_p) 
            admin_state = sx_port_admin_state_t_p_value(admin_state_t_p) 
            module_state = sx_port_module_state_t_p_value(module_state_t_p) 
            
            if port_mode == PORT_OPER_STATUS :
                if oper_state == SX_PORT_OPER_STATUS_UP :
                    #print "port state is up"
                    port_with_link_up_list.append(port)
            elif port_mode == PORT_ADMIN_STATUS :
                if admin_state == SX_PORT_ADMIN_STATUS_UP :
                    port_with_link_up_list.append(port)
    
    
    return port_with_link_up_list

def ip_addr_to_str(ip_addr):
    if ip_addr.version == SX_IP_VERSION_IPV4:
        return socket.inet_ntop(socket.AF_INET, struct.pack('!I', ip_addr.addr.ipv4.s_addr))
    else:
        bytes =  ip_addr.addr.ipv6.__in6_u.__u6_addr8
        byte_arr = []
        for i in xrange(0, 16):
            byte_arr.append(uint8_t_arr_getitem(bytes, i))
        if sys.byteorder == "little":
            dwords = struct.pack("!BBBBBBBBBBBBBBBB", byte_arr[3], byte_arr[2], byte_arr[1], byte_arr[0], byte_arr[7], byte_arr[6], byte_arr[5], byte_arr[4], byte_arr[11], byte_arr[10], byte_arr[9], byte_arr[8], byte_arr[15], byte_arr[14], byte_arr[13], byte_arr[12])
        else:
            dwords = struct.pack("!BBBBBBBBBBBBBBBB", byte_arr[0], byte_arr[1], byte_arr[2], byte_arr[3], byte_arr[4], byte_arr[5], byte_arr[6], byte_arr[7], byte_arr[8], byte_arr[9], byte_arr[10], byte_arr[11], byte_arr[12], byte_arr[13], byte_arr[14], byte_arr[15])
        return socket.inet_ntop(socket.AF_INET6, dwords)
    
def redirect_stdout():
    sys.stdout.flush()
    stdout_bak = os.dup(1)
    devnull = os.open('/dev/null', os.O_WRONLY)
    os.dup2(devnull, 1)
    os.close(devnull)
    return stdout_bak   

def print_all_attr_from_a_struct(struct):
    for attr in dir(struct):
        if not attr.startswith("__") and not attr == "this":
            print "%s = %s" % (attr, getattr(struct, attr)) 
            
def ip_prefix_to_str(ip_prefix):
    if ip_prefix.version == SX_IP_VERSION_IPV4:
        return socket.inet_ntop(socket.AF_INET, struct.pack('!I', ip_prefix.prefix.ipv4.addr.s_addr))
    else:
        bytes =  ip_prefix.prefix.ipv6.addr.__in6_u.__u6_addr8
        byte_arr = []
        for i in xrange(0, 16):
            byte_arr.append(uint8_t_arr_getitem(bytes, i))
        if sys.byteorder == "little":
            dwords = struct.pack("!BBBBBBBBBBBBBBBB", byte_arr[3], byte_arr[2], byte_arr[1], byte_arr[0], byte_arr[7], byte_arr[6], byte_arr[5], byte_arr[4], byte_arr[11], byte_arr[10], byte_arr[9], byte_arr[8], byte_arr[15], byte_arr[14], byte_arr[13], byte_arr[12])
        else:
            dwords = struct.pack("!BBBBBBBBBBBBBBBB", byte_arr[0], byte_arr[1], byte_arr[2], byte_arr[3], byte_arr[4], byte_arr[5], byte_arr[6], byte_arr[7], byte_arr[8], byte_arr[9], byte_arr[10], byte_arr[11], byte_arr[12], byte_arr[13], byte_arr[14], byte_arr[15])
        return socket.inet_ntop(socket.AF_INET6, dwords)  

def mask_len(ip_prefix):
    if ip_prefix.version == SX_IP_VERSION_IPV4:
        count = 0
        mask = ip_prefix.prefix.ipv4.mask.s_addr
        while mask != 0:
            if ((mask & 0x1) == 1):
                count = count + 1
            mask = mask >> 1
        return count
    else:
        bytes =  ip_prefix.prefix.ipv6.mask.__in6_u.__u6_addr8
        count = 0
        for i in xrange(0, 16):
            byte = uint8_t_arr_getitem(bytes, i)
            while byte != 0:
                if ((byte & 0x1) == 1):
                    count = count + 1
                byte = byte >> 1
        return count              
